#include <windows.h>
#include <windowsx.h>

#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glaux.h>
#include <math.h>
#include <stdio.h>

//#define _FULL_SCREEN		//         
#define MAX_STRING	1024

// Global variables
char			lpszAppName[]	= "Two Light";
char			lpszClassName[]	= "Two Light";
static	HGLRC	hRC;
static	HDC		hDC;
static	BOOL	m_bActive = FALSE;
GLuint			CUBE_LIST;
GLfloat			light0_Position[4] = {0.0f, 0.0f, 2.5f, 1.0f};
GLfloat			light1_Position[4] = {0.0f, 0.0f, -2.5f, 1.0f};
GLfloat			light1_Intensity[4] = {1.0f, 1.0f, 1.0f, 1.0f};

// Functions
GLvoid Initial(GLsizei Width, GLsizei Height);
GLvoid Resize(GLsizei Width, GLsizei Height);
GLvoid Draw(RECT* rect);
void DrawCube();
void CalculateList();
GLvoid StartRC();
GLvoid StopRC();
LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);


// Implementation
GLvoid Initial(GLsizei w, GLsizei h)
{
	Resize (w, h); 
}


GLvoid StartRC(void)
{
	glClearColor (0.5, 0.5, 0.75, 1.0);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_Intensity);

	CalculateList();
}

GLvoid StopRC(void)
{
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_COLOR_MATERIAL);

	glDeleteLists(CUBE_LIST, 1);
}

GLvoid Resize(GLsizei w, GLsizei h)
{
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	glFrustum (-1, 1, -1, 1, 4, 15);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();

 	glTranslatef(0.0, 0.0, -12.0);
	glRotatef(30.0, 1.0, 0.0, 0.0);
	glRotatef(70.0, 0.0, 1.0, 0.0);
}

void CalculateList()
{
	CUBE_LIST = glGenLists(1);
	glNewList(CUBE_LIST, GL_COMPILE);
		DrawCube();
	glEndList();
}

void DrawCube()
{
	glBegin (GL_QUADS);
		glNormal3f(0.0, 0.0, 1.0);
		glVertex3f (1.0, 1.0, 1.0);
		glVertex3f (-1.0, 1.0, 1.0);
		glVertex3f (-1.0, -1.0, 1.0);
		glVertex3f (1.0, -1.0, 1.0);
	glEnd();

	glBegin (GL_QUADS);
		glNormal3f(0.0, 0.0, -1.0);
		glVertex3f (1.0, 1.0, -1.0);
		glVertex3f (1.0, -1.0, -1.0);
		glVertex3f (-1.0, -1.0, -1.0);
		glVertex3f (-1.0, 1.0, -1.0);
	glEnd();

	glBegin (GL_QUADS);
		glNormal3f(-1.0, 0.0, 0.0);
		glVertex3f (-1.0, 1.0, 1.0);
		glVertex3f (-1.0, 1.0, -1.0);
		glVertex3f (-1.0, -1.0, -1.0);
		glVertex3f (-1.0, -1.0, 1.0);
	glEnd();

	glBegin (GL_QUADS);
		glNormal3f(1.0, 0.0, 0.0);
		glVertex3f (1.0, 1.0, 1.0);
		glVertex3f (1.0, -1.0, 1.0);
		glVertex3f (1.0, -1.0, -1.0);
		glVertex3f (1.0, 1.0, -1.0);
	glEnd();

	glBegin (GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f (-1.0, 1.0, -1.0);
		glVertex3f (-1.0, 1.0, 1.0);
		glVertex3f (1.0, 1.0, 1.0);
		glVertex3f (1.0, 1.0, -1.0);
	glEnd();

	glBegin(GL_QUADS);
		glNormal3f(0.0, -1.0, 0.0);
		glVertex3f (-1.0, -1.0, -1.0);
		glVertex3f (1.0, -1.0, -1.0);
		glVertex3f (1.0, -1.0, 1.0);
		glVertex3f (-1.0, -1.0, 1.0);
	glEnd();
}

GLvoid Draw()
{
	static GLfloat rot = 0.0f;

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	glColor3f (1.0, 1.0, 0.5);
	glCallList(CUBE_LIST);
	glRotatef(rot, 0.0f, 1.0f, 1.0f);

	glPushMatrix();
	glLightfv(GL_LIGHT0, GL_POSITION, light0_Position);
	glTranslatef(light0_Position[0],light0_Position[1],light0_Position[2]);
	glScalef(0.1f, 0.1f, 0.1f);
	glCallList(CUBE_LIST);
	glPopMatrix();

	glPushMatrix();
	glLightfv(GL_LIGHT1, GL_POSITION, light1_Position);
	glRotatef(rot, 1.0f, 0.0f, 0.0f);
	glTranslatef(light1_Position[0],light1_Position[1],light1_Position[2]);
	glScalef(0.1f, 0.1f, 0.1f);
	DrawCube();
	glPopMatrix();

	rot+=1.0f;
	glPopMatrix();

	SwapBuffers(hDC);
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	RECT Screen;
	GLuint PixelFormat;

	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),		// 
		1,									//  ?
		PFD_DRAW_TO_WINDOW|					// format must support Window
		PFD_SUPPORT_OPENGL|					// format must support OpenGL
		PFD_DOUBLEBUFFER,					// must support double buffer
		PFD_TYPE_RGBA,						//  RGBA 
		16,									// 16Bit color depth
		0, 0, 0, 0, 0, 0,					// Color bits ignored ?
		0,									// No Alpha buffer
		0,									// shift bit ignored
		0,									// no accumulation buffer
		0, 0, 0, 0,							// accumulation buffer bits ignored
		16,									// 16bit z-buffer (depth buffer)
		0,									// no stencil buffer
		0,									// no auxiliary buffer
		PFD_MAIN_PLANE,						// main drawing layer
		0,									// reserved
		0, 0, 0								// layer mask ignored
	};

	GetClientRect(hWnd, &Screen);

	switch(msg)
	{
	case WM_CREATE:
		hDC = GetDC(hWnd);
		PixelFormat = ChoosePixelFormat(hDC, &pfd);

		if(!PixelFormat){
			MessageBox(0, "Can't find suitable PixelFormat","Error", MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}

		if(!SetPixelFormat(hDC, PixelFormat, &pfd)) {
			MessageBox(0, "Can't set The PixelFormat","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}
		hRC = wglCreateContext(hDC);
		if(!hRC) {
			MessageBox(0, "Can't Create Render Device Context","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}
		if(!wglMakeCurrent(hDC, hRC)) {
			MessageBox(0, "Can't set current Render Device Context","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}

		Initial(Screen.right, Screen.bottom);
		StartRC();
		break;
	case WM_ACTIVATE:
		m_bActive = !((BOOL)HIWORD(wParam));
		break;
	case WM_CLOSE:
	case WM_DESTROY:
		StopRC();
		ChangeDisplaySettings(NULL, 0);

		wglMakeCurrent(hDC, NULL);
		wglDeleteContext(hRC);
		ReleaseDC(hWnd, hDC);

		PostQuitMessage(0);
		break;
	case WM_KEYDOWN:
		if(wParam == VK_ESCAPE) SendMessage(hWnd, WM_CLOSE, 0, 0);
		break;
	case WM_KEYUP:
		break;
	case WM_SIZE:
		Resize(LOWORD(lParam), HIWORD(lParam));
		break;
	default:
		return( DefWindowProc(hWnd, msg, wParam, lParam));
	}
	return(0);
}


int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmd, int nShowCmd)
{
	MSG			msg;
	WNDCLASS	wc;
	HWND		hWnd;

	wc.style		= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc	= (WNDPROC)WndProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInst;
	wc.hIcon		= NULL;
	wc.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= lpszClassName;

	if(!RegisterClass(&wc)) 
	{
		MessageBox(0,"Error Create window class","Error", MB_OK|MB_ICONERROR);
		return FALSE;
	}

#ifdef _FULL_SCREEN
	hWnd = CreateWindow(lpszClassName,
		lpszAppName,
		WS_POPUP |
		WS_CLIPCHILDREN |
		WS_CLIPSIBLINGS,
		0, 
		0, 
		GetSystemMetrics(SM_CXSCREEN), 
		GetSystemMetrics(SM_CYSCREEN), 
		NULL, 
		NULL, 
		hInst, 
		NULL);

	if(!hWnd){
		MessageBox(0,"Error Create window","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	}

	DEVMODE dmScreenSettings;

	memset(&dmScreenSettings, 0, sizeof(DEVMODE));
	dmScreenSettings.dmSize			= sizeof(DEVMODE);
	dmScreenSettings.dmPelsWidth	= GetSystemMetrics(SM_CXSCREEN);
	dmScreenSettings.dmPelsHeight	= GetSystemMetrics(SM_CYSCREEN);
	dmScreenSettings.dmFields		= DM_PELSWIDTH | DM_PELSHEIGHT;
	ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
#else
	hWnd = CreateWindow(lpszClassName,
		lpszAppName,
		WS_OVERLAPPEDWINDOW |
		WS_CLIPCHILDREN |
		WS_CLIPSIBLINGS,
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		NULL, 
		NULL, 
		hInst, 
		NULL);

	if(!hWnd){
		MessageBox(0,"Error Create window","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	}
#endif

	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);
	SetFocus(hWnd);

    while (TRUE)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                return msg.wParam;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else if (m_bActive)
        {
            Draw();
        }
        else
        {
            // Make sure we go to sleep if we have nothing else to do
            WaitMessage();
        }
    }
	return 0;
}

